#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
# Author: Ezra Kissel (kissel@es.net)
import os
import click
import json

from fabric_mgmt_cli.managecli.config_processor import ConfigProcessor
from fabric_mgmt_cli.managecli.net import services as NSOServices
from .nso import NSOClient
from .services import idipa, IDIPA_SERVICES
from .resources import Inventory

class NetCommand:
    PATH = os.environ.get('FABRIC_MGMT_CLI_CONFIG_PATH', './config.yml')

    def __init__(self, dry_run=False):
        self.cfg = ConfigProcessor(path=self.PATH)
        self.cfg.process()
        self.nso = NSOClient(self.cfg.get_net_url(),
                             self.cfg.get_net_username(),
                             self.cfg.get_net_password())
        self._res = None
        self._code = None
        self._dry_run = dry_run

    def sync_device(self, device, action):
        try:
            self._res, self._code = self.nso.sync(device, action)
        except:
            pass

    def get_devices(self):
        try:
            self._res, self._code = self.nso.devices()
        except:
            pass

    def get_packages(self):
        try:
            self._res, self._code = self.nso.packages()
        except:
            pass

    def get_services(self, service, name):
        try:
            self._res, self._code = self.nso.services(service, name)
        except:
            pass

    def get_config(self, device):
        try:
            self._res, self._code = self.nso.config(device)
        except:
            pass

    def create_service(self, service):
        if self._dry_run:
            self._res, self._code = service.json(), 0
            return
        try:
            self._res, self._code = self.nso.create_service(service)
        except:
            pass

    def delete_service(self, service, name):
        self._res, self._code = self.nso.delete_service(service, name)

    def print_result(self, txt=None, verbose=False):
        if self._res == None:
            return
        if txt:
            print (txt, end=f" - Response ({self._code}): ")
        if verbose:
            formatted_json = json.dumps(self._res, sort_keys=True, indent=4)
            try:
                from pygments import highlight, lexers, formatters
                colorful_json = highlight(formatted_json, lexers.JsonLexer(),
                                          formatters.TerminalFormatter())
                print (colorful_json)
            except:
                print (formatted_json)
            return

        new = dict()
        for k,v in self._res.items():
            if isinstance(v, dict):
                for l,w in v.items():
                    if not isinstance(w, list):
                        continue
                    for d in w:
                        if isinstance(d, dict) and "name" in d:
                            new[str(d['name'])] = d
                            d['opts'] = l
            elif isinstance(v, list):
                for d in v:
                    if isinstance(d, dict) and "name" in d:
                        new[str(d['name'])] = d

        for k,v in new.items():
            if "opts" in v:
                disp = f"{k: <30} | {v['opts']: <60} |"
            elif "description" in v:
                disp = f"{k: <30} | {v['description']: <60} |"
            else:
                disp = f"{k}"
            print (disp)

@click.group()
@click.pass_context
def net(ctx):
    """ Dataplane network management
    """
    config = os.getenv('FABRIC_MGMT_CLI_CONFIG_PATH')
    if config is None or config == "":
        ctx.fail('FABRIC_MGMT_CLI_CONFIG_PATH is not set')
    return

@click.group()
@click.option('-v', '--verbose', is_flag=True)
@click.pass_context
def show(ctx, verbose):
    """ Subgroup for network information commands
    """
    ctx.ensure_object(dict)
    ctx.obj['VERBOSE'] = verbose
    pass

@net.command()
@click.argument('action', nargs=1)
@click.option('--device', default=None, help='Device to sync', required=True)
@click.option('-v', '--verbose', is_flag=True)
@click.pass_context
def sync(ctx, verbose, device,  action):
    """ Control NSO device synchronization
    """
    sync_actions = ['sync-from', 'check-sync']
    if action not in sync_actions:
        print (f"Unknown action: {action}")
        return
    net_cmd = NetCommand()
    net_cmd.sync_device(device, action)
    net_cmd.print_result(verbose=verbose)

@net.command()
@click.option('--inventory', default=None, help='Inventory file', required=True)
@click.option('--epa', default=None, help='Endpoint A', required=True)
@click.option('--epz', default=None, help='Endpoint Z', required=False)
@click.option('--service', default=None, help='Service', required=True)
@click.option('--name', default=None, help='Service name', required=False)
@click.option('--dry-run', is_flag=True, help='Perform a dry run without configuring devices')
@click.pass_context
def create(ctx, inventory, epa, epz, service, name, dry_run):
    """ Create a new network service
    """
    try:
        inv = Inventory(inventory)
        a = inv.resolve_resource(epa)
        if service != "l3rt":
            z = inv.resolve_resource(epz)
        else:
            z = None
    except Exception as e:
        print (f"Error in create: {e}")
        return

    cls = getattr(NSOServices, service, None)
    if not cls:
        print (f"Unknown service {service}")
        return

    # name is auto-generated if None
    s = cls(name=name, epa=a, epz=z)
    net_cmd = NetCommand(dry_run)
    if service in IDIPA_SERVICES:
        ids = idipa(service, s.name)
        net_cmd.create_service(ids)
    net_cmd.create_service(s)
    net_cmd.print_result(f"Created service {service}: {s.name}", True)

@net.command()
@click.option('--service', default=None, help='Service', required=True)
@click.option('--name', default=None, help='Service name to delete', required=True)
@click.pass_context
def delete(ctx, service, name):
    """ Delete an existing network service by name
    """
    cls = getattr(NSOServices, service, None)
    if not cls:
        print (f"Unknown service {service}")
        return

    s = cls(name=name)
    net_cmd = NetCommand()
    if service in IDIPA_SERVICES:
        ids = idipa(service, s.name)
        net_cmd.delete_service(ids.service_str, name)
    net_cmd.delete_service(s.service_str, name)
    net_cmd.print_result(f"Delete {service} service: {s.name}", True)

@show.command()
@click.pass_context
def devices(ctx):
    """ Show devices known to network controller
    """
    net_cmd = NetCommand()
    net_cmd.get_devices()
    net_cmd.print_result(verbose=ctx.obj['VERBOSE'])

@show.command()
@click.pass_context
def packages(ctx):
    """ Show packages known to network controller
    """
    net_cmd = NetCommand()
    net_cmd.get_packages()
    net_cmd.print_result(verbose=ctx.obj['VERBOSE'])

@show.command()
@click.pass_context
@click.option('--service', default=None, help='Service', required=False)
@click.option('--name', default=None, help='Service name', required=False)
def services(ctx, service, name):
    """ Show services known to network controller
    """
    if name and not service:
        print ("Name requires specifying service option")
        return
    net_cmd = NetCommand()
    net_cmd.get_services(service, name)
    net_cmd.print_result(verbose=ctx.obj['VERBOSE'])

@show.command()
@click.option('--device', default=None, help='Device name', required=True)
@click.pass_context
def config(ctx, device):
    """ Show device configuration
    """
    net_cmd = NetCommand()
    net_cmd.get_config(device)
    net_cmd.print_result(verbose=True)

net.add_command(show)



import sys

class col:
    HEADER = '\033[35m'# PINK
    DIR = '\033[34m' # BLUE
    ITEM = '\033[37m' # LGRAY
    PROMPT = '\033[32m' # GREEN
    WARNING = '\033[33m' # YELLOW
    FAIL = '\033[31m' # RED
    ENDC = '\033[39m' # BLACK

class CText():
    def _color(self, c, e):
        print (c + str(e) + col.ENDC)
    def warn(self, e):
        self._color(col.WARNING, e)
    def error(self, e):
        self._color(col.FAIL, e)
    def item(self, e):
        self._color(col.ITEM, e)
    def info(self, e):
        print (str(e))



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)

import os
import click

from fabric_mgmt_cli.managecli.kafka_processor import KafkaProcessorSingleton
from fabric_mgmt_cli.managecli.manage_command import ManageCommand
from fabric_mgmt_cli.managecli.show_command import ShowCommand
from fabric_mgmt_cli.managecli.net import commands as netcommands


@click.group()
@click.option('-v', '--verbose', is_flag=True)
@click.pass_context
def managecli(ctx, verbose):
    ctx.ensure_object(dict)
    ctx.obj['VERBOSE'] = verbose


@click.group()
@click.pass_context
def slices(ctx):
    """ Slice management
    """
    config = os.getenv('FABRIC_MGMT_CLI_CONFIG_PATH')
    if config is None or config == "":
        ctx.fail('FABRIC_MGMT_CLI_CONFIG_PATH is not set')

    return


@slices.command()
@click.option('--sliceid', help='Slice Id', required=True)
@click.option('--actor', help='Actor Name', required=True)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def close(ctx, sliceid, actor, idtoken, refreshtoken):
    """ Closes slice for an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.close_slice(slice_id=sliceid, actor_name=actor,
                                 callback_topic=KafkaProcessorSingleton.get().get_callback_topic(), id_token=idtoken)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@slices.command()
@click.option('--sliceid', help='Slice Id', required=True)
@click.option('--actor', help='Actor Name', required=True)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def remove(ctx, sliceid, actor, idtoken, refreshtoken):
    """ Removes slice for an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.remove_slice(slice_id=sliceid, actor_name=actor,
                                  callback_topic=KafkaProcessorSingleton.get().get_callback_topic(), id_token=idtoken)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@slices.command()
@click.option('--email', help='User Email', required=True)
@click.option('--actor', help='Actor Name', required=True)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def removealldead(ctx, email, actor, idtoken, refreshtoken):
    """ Removes slice for an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.delete_dead_slices(email=email, actor_name=actor, id_token=idtoken,
                                        callback_topic=KafkaProcessorSingleton.get().get_callback_topic())
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@slices.command()
@click.option('--actor', default=None, help='Actor Name', required=True)
@click.option('--sliceid', default=None, help='Slice ID', required=False)
@click.option('--slicename', default=None, help='Slice Name', required=False)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.option('--email', default=None, help='User email', required=False)
@click.option('--states', help="Comma separated list of the states, possible values: "
                               "[nascent, configuring, stableok, stableerror, modifyok, modifyerror, closing, dead]",
                                required=False)
@click.option('--format', default='text', help='Output Format Type: text or json', required=False)
@click.pass_context
def query(ctx, actor, sliceid, slicename, idtoken, refreshtoken, email, states, format):
    """ Get slice(s) from an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ShowCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.get_slices(actor_name=actor, callback_topic=KafkaProcessorSingleton.get().get_callback_topic(),
                                slice_id=sliceid, slice_name=slicename, id_token=idtoken, email=email, states=states,
                                format=format)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))

@slices.command()
@click.option('--email', help='User Email', required=True)
@click.option('--actor', help='Actor Name', required=True)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def removealldead(ctx, email, actor, idtoken, refreshtoken):
    """ Removes slice for an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.delete_dead_slices(email=email, actor_name=actor, id_token=idtoken,
                                        callback_topic=KafkaProcessorSingleton.get().get_callback_topic())
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))

@click.group()
@click.pass_context
def slivers(ctx):
    """ Sliver management
    """
    config = os.getenv('FABRIC_MGMT_CLI_CONFIG_PATH')
    if config is None or config == "":
        ctx.fail('FABRIC_MGMT_CLI_CONFIG_PATH is not set')

    return


@slivers.command()
@click.option('--sliverid', help='Sliver Id', required=True)
@click.option('--actor', help='Actor Name', required=True)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def close(ctx, sliverid, actor, idtoken, refreshtoken):
    """ Closes sliver for an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.close_reservation(rid=sliverid, actor_name=actor,
                                       callback_topic=KafkaProcessorSingleton.get().get_callback_topic(), id_token=idtoken)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@slivers.command()
@click.option('--sliverid', help='Sliver Id', required=True)
@click.option('--actor', help='Actor Name', required=True)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def remove(ctx, sliverid, actor, idtoken, refreshtoken):
    """ Removes sliver for an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.remove_reservation(rid=sliverid, actor_name=actor,
                                        callback_topic=KafkaProcessorSingleton.get().get_callback_topic(), id_token=idtoken)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@slivers.command()
@click.option('--actor', help='Actor Name', required=True)
@click.option('--sliceid', default=None, help='Slice Id', required=False)
@click.option('--sliverid', default=None, help='Sliver Id', required=False)
@click.option('--states', default=None, help='Sliver State, Comma separated list of states, possible values: '
                                             '[nascent, ticketed, active, activeticketed, closed, closewait, '
                                             'failed, unknown, all]', required=False)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.option('--email', default=None, help='User Email', required=False)
@click.option('--site', default=None, help='Site Name', required=False)
@click.option('--type', default=None,
              help='Sliver Type, possible allowed values: '
                   '[VM, L2Bridge, L2STS, L2PTP, FABNetv4, FABNetv6, FABNetv4Ext, FABNetv6Ext, PortMirror, Facility, '
                   'L3VPN]',
              required=False)
@click.option('--format', default='text', help='Output Format Type: text or json', required=False)
@click.option('--fields', default=None, help='Comma separated list of fields to be displayed', required=False)
@click.pass_context
def query(ctx, actor, sliceid, sliverid, states, idtoken, refreshtoken, email, site, type, format, fields):
    """ Get sliver(s) from an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ShowCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.get_reservations(actor_name=actor,
                                      callback_topic=KafkaProcessorSingleton.get().get_callback_topic(),
                                      slice_id=sliceid, rid=sliverid, states=states, id_token=idtoken, email=email,
                                      site=site, type=type, format=format, fields=fields)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@click.group()
@click.pass_context
def delegations(ctx):
    """ Delegation management
    """
    config = os.getenv('FABRIC_MGMT_CLI_CONFIG_PATH')
    if config is None or config == "":
        ctx.fail('FABRIC_MGMT_CLI_CONFIG_PATH is not set')

    return


@delegations.command()
@click.option('--broker', help='Broker Name', required=True)
@click.option('--am', help='AM Name', required=True)
@click.option('--did', default=None, help='Delegation Id', required=False)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def claim(ctx, broker: str, am: str, did: str, idtoken, refreshtoken):
    """ Claim delegation(s) from AM to Broker
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.claim_delegations(broker=broker, am=am,
                                       callback_topic=KafkaProcessorSingleton.get().get_callback_topic(),
                                       did=did, id_token=idtoken)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@delegations.command()
@click.option('--broker', help='Broker Name', required=True)
@click.option('--am', help='AM Name', required=True)
@click.option('--did', default=None, help='Delegation Id', required=False)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def reclaim(ctx, broker: str, am: str, did: str, idtoken, refreshtoken):
    """ Reclaim delegation(s) from Broker to AM
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.reclaim_delegations(broker=broker, am=am,
                                         callback_topic=KafkaProcessorSingleton.get().get_callback_topic(),
                                         did=did, id_token=idtoken)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@delegations.command()
@click.option('--actor', help='Actor Name', required=True)
@click.option('--sliceid', default=None, help='Slice Id', required=False)
@click.option('--did', default=None, help='Delegation Id', required=False)
@click.option('--state',
              type=click.Choice(['nascent', 'delegated', 'reclaimed', 'closed', 'failed', 'all'],
                                case_sensitive=False),
              default='all', help='Sliver State', required=False)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.option('--format', default='text', help='Output Format Type: text or json', required=False)
@click.pass_context
def query(ctx, actor, sliceid, did, state, idtoken, refreshtoken, format):
    """ Get delegation(s) from an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ShowCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.get_delegations(actor_name=actor,
                                     callback_topic=KafkaProcessorSingleton.get().get_callback_topic(),
                                     slice_id=sliceid, did=did, state=state, id_token=idtoken, format=format)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@delegations.command()
@click.option('--did', help='Delegation Id', required=True)
@click.option('--actor', help='Actor Name', required=True)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def close(ctx, did, actor, idtoken, refreshtoken):
    """ Closes delegation for an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.close_delegation(did=did, actor_name=actor,
                                      callback_topic=KafkaProcessorSingleton.get().get_callback_topic(),
                                      id_token=idtoken)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@delegations.command()
@click.option('--did', help='Delegation Id', required=True)
@click.option('--actor', help='Actor Name', required=True)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def remove(ctx, did, actor, idtoken, refreshtoken):
    """ Removes sliver for an actor
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.remove_delegation(did=did, actor_name=actor,
                                       callback_topic=KafkaProcessorSingleton.get().get_callback_topic(),
                                       id_token=idtoken)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@click.group()
@click.pass_context
def maintenance(ctx):
    """ Maintenance Operations
    """
    config = os.getenv('FABRIC_MGMT_CLI_CONFIG_PATH')
    if config is None or config == "":
        ctx.fail('FABRIC_MGMT_CLI_CONFIG_PATH is not set')

    return


@maintenance.command()
@click.option('--actor', help='Actor Name', required=True)
@click.option('--mode', help='Mode value, i.e. PreMaint, Maint, Active', required=True)
@click.option('--projects', help='Comma separated list of Project Ids allowed to use TestBed in Maintenance mode',
              required=False, default=None)
@click.option('--users', help='Comma separated list of User emails allowed to use TestBed in Maintenance mode',
              required=False, default=None)
@click.option('--deadline',
              help='Start time that allows new resources to be created or extended up '
                   'until stated deadline in format: %Y-%m-%d %H:%M:%S %z',
              required=False, default=None)
@click.option('--end',
              help='Expected End for the Maintainenance in formt: %Y-%m-%d %H:%M:%S %z',
              required=False, default=None)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def testbed(ctx, actor: str, mode: str, projects: str, users: str, deadline: str, end: str, idtoken: str,
            refreshtoken: str):
    """ Change Maintenance modes (PreMaint, Maint, Active) for the Testbed
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.toggle_maintenance_mode(actor_name=actor,
                                             callback_topic=KafkaProcessorSingleton.get().get_callback_topic(),
                                             state=mode, projects=projects, users=users, id_token=idtoken,
                                             deadline=deadline, expected_end=end)
        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))


@maintenance.command()
@click.option('--actor', help='Actor Name', required=True)
@click.option('--name', help='Site Name', required=True)
@click.option('--mode', help='Mode value, i.e. PreMaint, Maint, Active', required=True)
@click.option('--projects', help='Comma separated list of Project Ids allowed to use TestBed in Maintenance mode',
              required=False, default=None)
@click.option('--users', help='Comma separated list of User emails allowed to use TestBed in Maintenance mode',
              required=False, default=None)
@click.option('--workers', help='Comma separated list of workers to be marked in Maintenance mode',
              required=False, default=None)
@click.option('--deadline',
              help='Start time that allows new resources to be created or extended up '
                   'until stated deadline in format: %Y-%m-%d %H:%M:%S %z',
              required=False, default=None)
@click.option('--end',
              help='Expected End for the Maintainenance in formt: %Y-%m-%d %H:%M:%S %z',
              required=False, default=None)
@click.option('--idtoken', default=None, help='Fabric Identity Token', required=False)
@click.option('--refreshtoken', default=None, help='Fabric Refresh Token', required=False)
@click.pass_context
def site(ctx, actor: str, name: str, mode: str, projects, users, workers: str, deadline: str, end: str,
         idtoken: str, refreshtoken: str):
    """ Change Maintenance modes (PreMaint, Maint, Active) for a specific Site or a specific worker
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(id_token=idtoken, refresh_token=refreshtoken, ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.toggle_maintenance_mode(actor_name=actor,
                                             callback_topic=KafkaProcessorSingleton.get().get_callback_topic(),
                                             state=mode, projects=projects, users=users, expected_end=end,
                                             site_name=name, workers=workers, deadline=deadline, id_token=idtoken)

        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))
'''

@maintenance.command()
@click.option('--actor', help='Actor Name', required=True)
@click.option('--site', help='Site Name', required=False)
@click.pass_context
def query(ctx, actor: str, site: str):
    """ Query Maintenance Status for Testbed/Site
    """
    try:
        idtoken = KafkaProcessorSingleton.get().start(ignore_tokens=True)
        mgmt_command = ManageCommand(logger=KafkaProcessorSingleton.get().logger)
        mgmt_command.get_maintenance_mode(actor_name=actor,
                                          callback_topic=KafkaProcessorSingleton.get().get_callback_topic(),
                                          site=site)

        KafkaProcessorSingleton.get().stop()
    except Exception as e:
        # traceback.print_exc()
        click.echo('Error occurred: {}'.format(e))
'''

managecli.add_command(slices)
managecli.add_command(slivers)
managecli.add_command(delegations)
managecli.add_command(maintenance)
managecli.add_command(netcommands.net)



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import traceback
from datetime import datetime, timezone
from typing import Tuple

from fabric_cf.actor.core.apis.abc_delegation import DelegationState
from fabric_cf.actor.core.common.constants import Constants
from fabric_cf.actor.core.kernel.slice_state_machine import SliceState
from fabric_cf.actor.core.manage.error import Error
from fabric_cf.actor.core.util.id import ID
from fabric_mb.message_bus.messages.delegation_avro import DelegationAvro
from fabric_mb.message_bus.messages.site_avro import SiteAvro
from fabric_mb.message_bus.messages.slice_avro import SliceAvro
from fim.slivers.maintenance_mode import MaintenanceInfo, MaintenanceEntry, MaintenanceState

from fabric_mgmt_cli.managecli.show_command import ShowCommand


class ManageCommand(ShowCommand):
    def do_close_reservation(self, *, rid: str, actor_name: str, callback_topic: str,
                             id_token: str) -> Tuple[bool, Error]:
        """
        Close reservation by invoking Management Actor Close reservation API
        @param rid reservation id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        @return Tuple[bool, Error] indicating success or failure status and error containing failure details
        """
        actor = self.get_actor(actor_name=actor_name)
        if actor is None:
            raise Exception(f"Invalid arguments actor_name {actor_name} not found")

        try:
            actor.prepare(callback_topic=callback_topic)
            reservation_id = ID(uid=rid) if rid is not None else None
            return actor.close_reservation(rid=reservation_id), actor.get_last_error()
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

        return False, actor.get_last_error()

    def close_reservation(self, *, rid: str, actor_name: str, callback_topic: str, id_token: str):
        """
        Close reservation
        @param rid reservation id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        """
        try:
            result, error = self.do_close_reservation(rid=rid, actor_name=actor_name,
                                                      callback_topic=callback_topic, id_token=id_token)
            print(result)
            if result is False:
                self.print_result(status=error.get_status())
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

    def do_close_slice(self, *, slice_id: ID, actor_name: str, callback_topic: str,
                       id_token: str) -> Tuple[bool, Error]:
        """
        Close slice by invoking Management Actor Close reservations API
        @param slice_id slice id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        @return Tuple[bool, Error] indicating success or failure status and error containing failure details
        """
        actor = self.get_actor(actor_name=actor_name)
        if actor is None:
            raise Exception("Invalid arguments actor_name {} not found".format(actor_name))

        try:
            actor.prepare(callback_topic=callback_topic)
            return actor.close_reservations(slice_id=slice_id), actor.get_last_error()
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

        return False, actor.get_last_error()

    def close_slice(self, *, slice_id: str, actor_name: str, callback_topic: str, id_token: str):
        """
        Close slice
        @param slice_id slice id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        """
        try:
            sid = None
            if slice_id is not None:
                sid = ID(uid=slice_id)
            result, error = self.do_close_slice(slice_id=sid, actor_name=actor_name,
                                                callback_topic=callback_topic, id_token=id_token)
            print(result)
            if result is False:
                self.print_result(status=error.get_status())
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

    def do_remove_reservation(self, *, rid: str, actor_name: str, callback_topic: str,
                              id_token: str) -> Tuple[bool, Error]:
        """
        Remove reservation by invoking Management Actor Remove reservation API
        @param rid reservation id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        @return Tuple[bool, Error] indicating success or failure status and error containing failure details
        """
        actor = self.get_actor(actor_name=actor_name)
        if actor is None:
            raise Exception("Invalid arguments actor_name {} not found".format(actor_name))

        try:
            actor.prepare(callback_topic=callback_topic)
            reservation_id = ID(uid=rid) if rid is not None else None
            return actor.remove_reservation(rid=reservation_id), actor.get_last_error()
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())
        return False, actor.get_last_error()

    def remove_reservation(self, *, rid: str, actor_name: str, callback_topic: str, id_token: str):
        """
        Remove reservation
        @param rid reservation id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        """
        try:
            result, error = self.do_remove_reservation(rid=rid, actor_name=actor_name, callback_topic=callback_topic,
                                                       id_token=id_token)
            print(result)
            if result is False:
                self.print_result(status=error.get_status())
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

    def do_remove_slice(self, *, slice_id: str, actor_name: str, callback_topic: str,
                        id_token: str) -> Tuple[bool, Error]:
        """
        Remove slice by invoking Management Actor remove slice API
        @param slice_id slice id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        @return Tuple[bool, Error] indicating success or failure status and error containing failure details
        """
        actor = self.get_actor(actor_name=actor_name)
        if actor is None:
            raise Exception("Invalid arguments actor_name {} not found".format(actor_name))

        try:
            actor.prepare(callback_topic=callback_topic)
            sid = ID(uid=slice_id) if slice_id is not None else None
            return actor.remove_slice(slice_id=sid), actor.get_last_error()
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())
        return False, actor.get_last_error()

    def remove_slice(self, *, slice_id: str, actor_name: str, callback_topic: str, id_token: str):
        """
        Remove slice
        @param slice_id slice id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        """
        try:
            result, error = self.do_remove_slice(slice_id=slice_id, actor_name=actor_name,
                                                 callback_topic=callback_topic, id_token=id_token)
            print(result)
            if result is False:
                self.print_result(status=error.get_status())
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

    def do_claim_delegations(self, *, broker: str, am_guid: ID, callback_topic: str, id_token: str = None,
                             did: str = None) -> Tuple[DelegationAvro, Error]:
        """
        Claim delegations by invoking Management Actor Claim Delegations API
        @param broker broker guid
        @param am_guid am guid
        @param callback_topic callback topic
        @param id_token id token
        @param did delegation id
        @return Tuple[Delegation, Error] Delegation on success and Error in case of failure
        """
        actor = self.get_actor(actor_name=broker)

        if actor is None:
            raise Exception("Invalid arguments actor {} not found".format(broker))
        try:
            actor.prepare(callback_topic=callback_topic)

            dlg = actor.claim_delegations(broker=am_guid, did=did)
            return dlg, actor.get_last_error()
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

        return None, actor.get_last_error()

    def claim_delegations(self, *, broker: str, am: str, callback_topic: str, did: str = None, id_token: str = None):
        """
        Claim delegations
        @param broker broker name
        @param am am name
        @param callback_topic callback topic
        @param id_token id token
        @param did delegation id
        """
        try:
            am_actor = self.get_actor(actor_name=am)
            broker_actor = self.get_actor(actor_name=broker)

            if am_actor is None or broker_actor is None:
                raise Exception("Invalid arguments am_actor {} or broker_actor {} not found".format(am_actor,
                                                                                                    broker_actor))
            delegations = []
            if did is None:
                delegations, error = self.do_get_delegations(actor_name=am, callback_topic=callback_topic, did=did,
                                                             id_token=id_token)
                if delegations is None:
                    print("Error occurred while getting delegations for actor: {}".format(am))
                    self.print_result(status=error.get_status())
                    return

                if delegations is None or len(delegations) == 0:
                    print("No delegations to be claimed from {} by {}:".format(am, broker))
                    return
            else:
                dd = DelegationAvro()
                dd.slice = SliceAvro()
                dd.slice.slice_name = broker
                dd.delegation_id = did
                delegations = [dd]

            claimed = False
            for d in delegations:
                if d.get_state() == DelegationState.Failed.value or d.get_state() == DelegationState.Closed.value:
                    continue
                if d.get_slice_object().get_slice_name() == broker:
                    print("Claiming Delegation# {}".format(d.get_delegation_id()))
                    delegation, error = self.do_claim_delegations(broker=broker, am_guid=am_actor.get_guid(),
                                                                  did=d.get_delegation_id(), callback_topic=callback_topic,
                                                                  id_token=id_token)
                    claimed = True
                    if delegation is not None:
                        print("Delegation claimed: {} ".format(delegation.get_delegation_id()))
                    else:
                        self.print_result(status=error.get_status())
            if not claimed:
                print(f"No delegations found for Broker# {broker}")
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

    def do_reclaim_delegations(self, *, broker: str, am_guid: ID, callback_topic: str, id_token: str = None,
                               did: str = None) -> Tuple[DelegationAvro, Error]:
        """
        Reclaim delegations by invoking Management Actor Claim Delegations API
        @param broker broker guid
        @param am_guid am guid
        @param callback_topic callback topic
        @param id_token id token
        @param did delegation id
        @return Tuple[Delegation, Error] Delegation on success and Error in case of failure
        """
        actor = self.get_actor(actor_name=broker)

        if actor is None:
            raise Exception("Invalid arguments actor {} not found".format(broker))
        try:
            actor.prepare(callback_topic=callback_topic)

            dlg = actor.reclaim_delegations(broker=am_guid, did=did)
            return dlg, actor.get_last_error()
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

        return None, actor.get_last_error()

    def reclaim_delegations(self, *, broker: str, am: str, callback_topic: str, did: str = None, id_token: str = None):
        """
        Claim delegations
        @param broker broker name
        @param am am name
        @param callback_topic callback topic
        @param id_token id token
        @param did delegation id
        """
        try:
            am_actor = self.get_actor(actor_name=am)
            broker_actor = self.get_actor(actor_name=broker)

            if am_actor is None or broker_actor is None:
                raise Exception("Invalid arguments am_actor {} or broker_actor {} not found".format(am_actor,
                                                                                                    broker_actor))

            delegations, error = self.do_get_delegations(actor_name=am, callback_topic=callback_topic, did=did,
                                                         id_token=id_token)
            if delegations is None:
                print("Error occurred while getting delegations for actor: {}".format(am))
                self.print_result(status=error.get_status())
                return

            if delegations is None or len(delegations) == 0:
                print("No delegations to be reclaimed from {} by {}:".format(am, broker))
                return

            claimed = False
            for d in delegations:
                if d.get_state() == DelegationState.Failed.value or d.get_state() == DelegationState.Closed.value:
                    continue
                if d.get_slice_object().get_slice_name() == broker:
                    print("Reclaiming Delegation# {}".format(d.get_delegation_id()))
                    delegation, error = self.do_reclaim_delegations(broker=broker, am_guid=am_actor.get_guid(),
                                                                    did=d.get_delegation_id(),
                                                                    callback_topic=callback_topic,
                                                                    id_token=id_token)
                    claimed = True
                    if delegation is not None:
                        print("Delegation reclaimed: {} ".format(delegation.get_delegation_id()))
                    else:
                        self.print_result(status=error.get_status())
            if not claimed:
                print(f"No delegations found for Broker# {broker}")
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

    def toggle_maintenance_mode(self, *, actor_name: str, callback_topic: str, state: str, projects: str = None,
                                users: str = None, site_name: str = None, workers: str = None, deadline: str = None,
                                expected_end: str = None, id_token: str = None):
        """
        Toggle Maintenance Mode
        @param actor_name actor name
        @param callback_topic callback topic
        @param state Maintenance State
        @param projects comma separated list of project_ids allowed in maintenance
        @param users comma separated list of email address of the users allowed in maintenance
        @param site_name site name
        @param workers comma separated list of specific workers on a site which are in maintenance
        @param deadline start time for the Maintenance
        @param expected_end Expected End time for the Maintenance
        @param id_token id token
        """
        status = False
        error = ""
        try:
            actor = self.get_actor(actor_name=actor_name)

            if actor is None:
                raise Exception(f"Invalid arguments! {actor_name} not found")

            mode = MaintenanceState.from_string(state)

            if mode == MaintenanceState.PreMaint and deadline is None:
                raise Exception("Required parameter: deadline is missing!")

            if deadline is not None:
                try:
                    maint_start_time = datetime.fromisoformat(deadline)
                except Exception as e:
                    raise Exception(f"Deadline is not in ISO 8601 format")
                now = datetime.now(timezone.utc)
                if maint_start_time < now:
                    raise Exception(f"Deadline cannot be before current time {now}")

            if expected_end is not None:
                try:
                    expected_end_time = datetime.fromisoformat(deadline)
                except Exception as e:
                    raise Exception(f"Expected end is not in ISO 8601 format")
                now = datetime.now(timezone.utc)
                if expected_end_time < now:
                    raise Exception(f"Expected end cannot be before current time {now}")

            try:
                actor.prepare(callback_topic=callback_topic)
                sites = None
                if site_name is None:
                    site_name = Constants.ALL

                worker_list = [site_name]
                if workers is not None:
                    worker_list = workers.split(",")

                maint_info = MaintenanceInfo()
                for w in worker_list:
                    entry = MaintenanceEntry(state=state, deadline=deadline, expected_end=expected_end)
                    maint_info.add(w, entry)
                site_avro = SiteAvro(name=site_name, maint_info=maint_info)
                sites = [site_avro]

                status = actor.toggle_maintenance_mode(actor_guid=str(actor.get_guid()), sites=sites, projects=projects,
                                                       users=users, callback_topic=callback_topic)

            except Exception as e:
                self.logger.error(f"Exception occurred e: {e}")
                self.logger.error(traceback.format_exc())

            error = actor.get_last_error()
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())
            error = str(e)

        if status:
            print(f"Maintenance mode successfully set to {state}")
        else:
            print(f"Failure to set maintenance mode: [{state}]; Error: [{error}]")

    def delete_dead_slices(self, *, actor_name: str, callback_topic: str, id_token: str, email: str):

        try:
            slices, error = self.do_get_slices(actor_name=actor_name, callback_topic=callback_topic, id_token=id_token,
                                               email=email, slice_name=None)

            if slices is not None:
                for s in slices:
                    state = SliceState(s.get_state())
                    if state not in [SliceState.Closing, SliceState.Dead]:
                        continue
                    print(f"Attempting to remove reservations for slice {s.get_slice_id()} in state {state}")
                    reservations, error = self.do_get_reservations(actor_name=actor_name, callback_topic=callback_topic,
                                                                   id_token=id_token, slice_id=s.get_slice_id())

                    if reservations is None:
                        print(f"No reservations to remove for slice {s.get_slice_id()}")
                    else:
                        for r in reservations:
                            print(f"Attempting to remove reservation: {r.get_reservation_id()}")
                            self.do_remove_reservation(rid=r.get_reservation_id(), actor_name=actor_name,
                                                       callback_topic=callback_topic, id_token=id_token)

                    print(f"Attempting to remove slice: {s.get_slice_id()}")
                    self.remove_slice(slice_id=s.get_slice_id(), actor_name=actor_name,
                                      callback_topic=callback_topic, id_token=id_token)
            else:
                print("No Dead/closing slices to remove")
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

    def do_close_delegation(self, *, did: str, actor_name: str, callback_topic: str,
                            id_token: str) -> Tuple[bool, Error]:
        """
        Close delegation by invoking Management Actor Close delegation API
        @param did delegation id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        @return Tuple[bool, Error] indicating success or failure status and error containing failure details
        """
        actor = self.get_actor(actor_name=actor_name)
        if actor is None:
            raise Exception(f"Invalid arguments actor_name {actor_name} not found")

        try:
            actor.prepare(callback_topic=callback_topic)
            return actor.close_delegation(did=did), actor.get_last_error()
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

        return False, actor.get_last_error()

    def close_delegation(self, *, did: str, actor_name: str, callback_topic: str, id_token: str):
        """
        Close delegation
        @param did delegation id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        """
        try:
            result, error = self.do_close_delegation(did=did, actor_name=actor_name,
                                                     callback_topic=callback_topic, id_token=id_token)
            print(result)
            if result is False:
                self.print_result(status=error.get_status())
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())

    def do_remove_delegation(self, *, did: str, actor_name: str, callback_topic: str,
                             id_token: str) -> Tuple[bool, Error]:
        """
        Remove delegation by invoking Management Actor Remove delegation API
        @param did delegation id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        @return Tuple[bool, Error] indicating success or failure status and error containing failure details
        """
        actor = self.get_actor(actor_name=actor_name)
        if actor is None:
            raise Exception("Invalid arguments actor_name {} not found".format(actor_name))

        try:
            actor.prepare(callback_topic=callback_topic)
            return actor.remove_delegation(did=did), actor.get_last_error()
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())
        return False, actor.get_last_error()

    def remove_delegation(self, *, did: str, actor_name: str, callback_topic: str, id_token: str):
        """
        Remove delegation
        @param did delegation id
        @param actor_name actor name
        @param callback_topic callback topic
        @param id_token identity token
        """
        try:
            result, error = self.do_remove_delegation(did=did, actor_name=actor_name, callback_topic=callback_topic,
                                                      id_token=id_token)
            print(result)
            if result is False:
                self.print_result(status=error.get_status())
        except Exception as e:
            self.logger.error(f"Exception occurred e: {e}")
            self.logger.error(traceback.format_exc())


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import unittest

from click.testing import CliRunner
from fabric_mgmt_cli.managecli import managecli
import os


class ManageCliTest(unittest.TestCase):
    am_slice_id = 'b0176076-9fce-4084-88b0-490f22425af1'
    broker_slice_id = 'test_broker'
    am_res_id = 'test_res_am'
    broker_res_id = 'test_res_broker'
    os.environ['FABRIC_MGMT_CLI_CONFIG_PATH'] = "."

    def test_d_get_slices_am(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slices', 'query', '--actor', 'site1-am'])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find("Slice Guid") != -1)

    def test_e_get_slices_broker(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slices', 'query', '--actor', 'broker'])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find("Slice Guid") != -1)

    def test_f_get_slice_am(self):
        print("Using slice_id:{}".format(self.am_slice_id))
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slices', 'query', '--actor', 'site1-am', '--sliceid',
                                                     ManageCliTest.am_slice_id])
        print("Result: {}".format(result.output))
        #self.assertTrue(result.output.find("ErrorNoSuchSlice") != -1)

    def test_g_get_slice_broker(self):
        print("Using slice_id:{}".format(self.broker_slice_id))
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slices', 'query', '--actor', 'broker', '--sliceid',
                                                     ManageCliTest.broker_slice_id])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find("ErrorNoSuchSlice") != -1)

    def test_h_get_reservations_broker(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slivers', 'query', '--actor', 'broker'])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find("Reservation ID") != -1)

    def test_i_get_reservations_am(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slivers', 'query', '--actor', 'site1-am'])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find("Reservation ID") != -1)

    def test_j_get_reservation(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slivers', 'query', '--actor', 'broker', '--rid',
                                                     ManageCliTest.broker_res_id])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find("ErrorNoSuchReservation") != -1)

    def test_k_get_reservation(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slivers', 'query', '--actor', 'site1-am', '--rid',
                                                     ManageCliTest.am_res_id])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find("ErrorNoSuchReservation") != -1)

    def test_l_close_reservation(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slivers', 'close', '--actor', 'broker', '--rid',
                                                     ManageCliTest.am_res_id])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find('False') != -1)

    def test_m_close_slice(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slices', 'close', '--actor', 'broker', '--sliceid',
                                                     ManageCliTest.broker_slice_id])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find('False') != -1)

    def test_n_remove_reservation(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slivers', 'remove', '--actor', 'broker', '--rid',
                                                     ManageCliTest.am_res_id])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find('True') != -1)

    def test_o_remove_slice(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['slices', 'remove', '--actor', 'broker', '--sliceid',
                                                     ManageCliTest.broker_slice_id])
        print("Result: {}".format(result.output))
        self.assertTrue(result.output.find('True') != -1)

    def test_p_claim_resources(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['delegations', 'claim', '--broker', 'broker', '--am',
                                                     'site1-am'])
        print("Result: {}".format(result.output))
        #self.assertTrue(result.output.find("Code") == -1)

    def test_q_get_delegations_am(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['delegations', 'show', '--actor', 'site1-am'])
        print(result.output)
        #self.assertTrue(result.exit_code != 0)

    def test_r_get_delegations_am_id_token(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['delegations', 'show', '--actor', 'site1-am', '--idtoken', 'eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJlbWFpbCI6Imt0aGFyZTEwQGVtYWlsLnVuYy5lZHUiLCJnaXZlbl9uYW1lIjoiS29tYWwiLCJmYW1pbHlfbmFtZSI6IlRoYXJlamEiLCJuYW1lIjoiS29tYWwgVGhhcmVqYSIsImlzcyI6Imh0dHBzOi8vY2lsb2dvbi5vcmciLCJzdWIiOiJodHRwOi8vY2lsb2dvbi5vcmcvc2VydmVyQS91c2Vycy8xMTkwNDEwMSIsImF1ZCI6ImNpbG9nb246L2NsaWVudF9pZC8xMjUzZGVmYzYwYTMyM2ZjYWEzYjQ0OTMyNjQ3NjA5OSIsInRva2VuX2lkIjoiaHR0cHM6Ly9jaWxvZ29uLm9yZy9vYXV0aDIvaWRUb2tlbi8zZjVkOWZmMTBlMzJiZTU3YjMzY2MwY2U5ZmI3OWE2Yy8xNjA0NTQ4NTY0MzE3IiwiYXV0aF90aW1lIjoiMTYwNDU0ODU2NCIsImV4cCI6MTYwNDU1MjE3MCwiaWF0IjoxNjA0NTQ4NTcwLCJyb2xlcyI6WyJDTzptZW1iZXJzOmFjdGl2ZSIsIkNPOkNPVTpKdXB5dGVyaHViOm1lbWJlcnM6YWN0aXZlIiwiQ086Q09VOnByb2plY3QtbGVhZHM6bWVtYmVyczphY3RpdmUiXSwic2NvcGUiOiJhbGwiLCJwcm9qZWN0IjoiYWxsIn0.CYIwHmz6k83OQaCZYYlQr-idK-7xdfFo3iLpw2nic7QkfREhmEMT5HPLUvKMuXDWteOtYqa62pMDsFynaZ6YQWW6r-1WzU57Xmff8uxrQwkNTe8Jgm6GVEEfRLaEXAjoRN1VfUWflMUOa2PLTPisIBtgbi9jqqUwJpZYiSgytIFr3amLwPKrdZVx0Hsx0ldycPzqeTgry7zLWQFF4WMFiRRw5dj43ma9Tzm34FXUE6_ZX5cmPuTpu7NWftrWIsSXdYtR81MIpG9F5a7bvSLyVxRKAqGxNEbELgkuANGSy9SlfMTZ3owBTzg_Rx1WJGhNRRTFlCP7MEbigjvrquNylg'])
        print(result.output)
        #self.assertTrue(result.exit_code != 0)


    def test_s_get_delegations_am_refresh_token(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['delegations', 'delegations', '--actor', 'site1-am',
                                                     '--refreshtoken', 'https://cilogon.org/oauth2/refreshToken/592aeceab9fa39bff826ba3d829262c9/1604548564478'])
        print(result.output)
        #self.assertTrue(result.exit_code != 0)

    def test_q_get_delegations_broker(self):
        runner = CliRunner()
        result = runner.invoke(managecli.managecli, ['show', 'delegations', '--actor', 'broker'])
        print(result.output)
        #self.assertTrue(result.exit_code != 0)


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)

import yaml
from fabric_cf.actor.core.common.constants import Constants

from fabric_mgmt_cli.managecli.configuration import Configuration
from fabric_mb.message_bus.messages.auth_avro import AuthAvro


class ConfigProcessor:
    """
    Configuration Processor to read and load config
    """
    def __init__(self, *, path: str = None):
        self.path = path
        self.config = None
        self.auth = None

    def process(self, *, config: Configuration = None):
        if config is None:
            self.read_configuration()
        else:
            self.config = config
        if self.config.get_auth() is not None:
            self.auth = AuthAvro()
            self.auth.name = self.config.get_auth().get_name()
            self.auth.guid = self.config.get_auth().get_guid()

    def read_configuration(self) -> Configuration:
        if self.path is None:
            raise Exception("No data source has been specified")
        config_dict = None
        with open(self.path) as f:
            config_dict = yaml.safe_load(f)
        self.config = Configuration(config_dict)
        return self.config

    def get_auth(self) -> AuthAvro:
        return self.auth

    def get_kafka_server(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_kafka_server()
        else:
            return None

    def get_kafka_schema_registry(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_kafka_schema_registry()
        else:
            return None

    def get_kafka_key_schema(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_kafka_key_schema()
        else:
            return None

    def get_kafka_value_schema(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_kafka_value_schema()
        else:
            return None

    def get_kafka_topic(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_kafka_topic()
        else:
            return None

    def get_security_protocol(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_security_protocol()
        return None

    def get_group_id(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_group_id()
        return None

    def get_ca_location(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_ca_location()
        return None

    def get_cert_location(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_cert_location()
        return None

    def get_key_location(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_key_location()
        return None

    def get_key_password(self) -> str:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_key_password()
        return None

    def get_peers(self) -> list:
        return self.config.get_peers()

    def get_log_dir(self) -> str:
        if self.config is not None and self.config.get_logging() is not None:
            return self.config.get_logging().get_log_dir()
        return None

    def get_log_file(self) -> str:
        if self.config is not None and self.config.get_logging() is not None:
            return self.config.get_logging().get_log_file()
        return None

    def get_log_level(self):
        if self.config is not None and self.config.get_logging() is not None:
            return self.config.get_logging().get_log_level()
        return None

    def get_log_retain(self) -> int:
        if self.config is not None and self.config.get_logging() is not None:
            return int(self.config.get_logging().get_log_retain())
        return None

    def get_log_size(self) -> int:
        if self.config is not None and self.config.get_logging() is not None:
            return int(self.config.get_logging().get_log_size())
        return None

    def get_log_name(self) -> str:
        if self.config is not None and self.config.get_logging() is not None:
            return self.config.get_logging().get_log_name()
        return None

    def get_kafka_config(self) -> dict:
        if self.config is not None and self.config.get_runtime_config() is not None:
            return self.config.get_runtime_config().get_kafka_config()
        return None

    def get_kafka_config_producer(self) -> dict:
        if self.config is None or self.config.get_runtime_config() is None:
            return None

        bootstrap_server = self.get_kafka_server()
        schema_registry = self.get_kafka_schema_registry()
        security_protocol = self.get_security_protocol()
        ssl_ca_location = self.get_ca_location()
        ssl_certificate_location = self.get_cert_location()
        ssl_key_location = self.get_key_location()
        ssl_key_password = self.get_key_password()

        conf = {Constants.BOOTSTRAP_SERVERS: bootstrap_server,
                Constants.SECURITY_PROTOCOL: security_protocol,
                Constants.SSL_CA_LOCATION: ssl_ca_location,
                Constants.SSL_CERTIFICATE_LOCATION: ssl_certificate_location,
                Constants.SSL_KEY_LOCATION: ssl_key_location,
                Constants.SSL_KEY_PASSWORD: ssl_key_password,
                Constants.SCHEMA_REGISTRY_URL: schema_registry}

        sasl_username = self.config.get_runtime_config().get_kafka_config().get(
            Constants.PROPERTY_CONF_KAFKA_SASL_PRODUCER_USERNAME, None)

        sasl_password = self.config.get_runtime_config().get_kafka_config().get(
            Constants.PROPERTY_CONF_KAFKA_SASL_PRODUCER_PASSWORD, None)

        sasl_mechanism = self.config.get_runtime_config().get_kafka_config().get(
            Constants.PROPERTY_CONF_KAFKA_SASL_MECHANISM, None)

        if sasl_username is not None and sasl_username != '' and sasl_password is not None and sasl_password != '':
            conf[Constants.SASL_USERNAME] = sasl_username
            conf[Constants.SASL_PASSWORD] = sasl_password
            conf[Constants.SASL_MECHANISM] = sasl_mechanism

        return conf

    def get_kafka_config_consumer(self) -> dict:
        if self.config is None or self.config.get_runtime_config() is None:
            return None

        conf = self.get_kafka_config_producer()
        conf['auto.offset.reset'] = 'earliest'

        group_id = self.get_group_id()
        conf[Constants.GROUP_ID] = group_id

        sasl_username = self.config.get_runtime_config().get_kafka_config().get(
            Constants.PROPERTY_CONF_KAFKA_SASL_CONSUMER_USERNAME, None)
        sasl_password = self.config.get_runtime_config().get_kafka_config().get(
            Constants.PROPERTY_CONF_KAFKA_SASL_CONSUMER_PASSWORD, None)

        if sasl_username is not None and sasl_username != '' and sasl_password is not None and sasl_password != '':
            conf[Constants.SASL_USERNAME] = sasl_username
            conf[Constants.SASL_PASSWORD] = sasl_password
        return conf

    def get_credmgr_host(self) -> str:
        return self.config.get_auth().get_credmgr_host()

    def get_net_url(self) -> str:
        if self.config is not None and self.config.get_net() is not None:
            return self.config.get_net().get_url()
        return None

    def get_net_username(self) -> str:
        if self.config is not None and self.config.get_net() is not None:
            return self.config.get_net().get_username()
        return None

    def get_net_password(self) -> str:
        if self.config is not None and self.config.get_net() is not None:
            return self.config.get_net().get_password()
        return None

    def get_net_validate_certs(self) -> str:
        if self.config is not None and self.config.get_net() is not None:
            return self.config.get_net().get_validate_certs()
        return None



import json
import requests

import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

from .util import CText


cout = CText()

class NSOClient():
    def __init__(self, url, user, pwd):
        self._user = user
        self._pwd = pwd
        self._url = url

    def _get(self, ep):
        hdr = {"Accept": "application/yang-data+json"}
        url = f"{self._url}/{ep}"
        try:
            ret = requests.get(url, auth=(self._user, self._pwd), headers=hdr, verify=False)
            if not ret.text:
                return (dict(), ret.status_code)
            return (ret.json(), ret.status_code)
        except Exception as e:
            cout.error(f"GET: {e}")
            raise e

    def _patch(self, ep, data=None):
        hdr = {"Accept": "application/yang-data+json",
               "Content-type": "application/yang-data+json"}
        url = f"{self._url}/{ep}"
        try:
            ret = requests.patch(url, auth=(self._user, self._pwd), headers=hdr,
                                 data=data, verify=False)
            if not ret.text:
                return (dict(), ret.status_code)
            return (ret.json(), ret.status_code)
        except Exception as e:
            cout.error(f"PATCH: {e}")
            raise e

    def _post(self, ep, data=None):
        hdr = {"Accept": "application/yang-data+json",
               "Content-type": "application/yang-data+json"}
        url = f"{self._url}/{ep}"
        try:
            ret = requests.post(url, auth=(self._user, self._pwd), headers=hdr,
                                data=data, verify=False)
            if not ret.text:
                return (dict(), ret.status_code)
            return (ret.json(), ret.status_code)
        except Exception as e:
            cout.error(f"POST: {e}")
            raise e

    def _delete(self, ep, data=None):
        hdr = {"Accept": "application/yang-data+json",
               "Content-type": "application/yang-data+json"}
        url = f"{self._url}/{ep}"
        try:
            ret = requests.delete(url, auth=(self._user, self._pwd), headers=hdr,
                                  data=data, verify=False)
            if not ret.text:
                return (dict(), ret.status_code)
            return (ret.json(), ret.status_code)
        except Exception as e:
            cout.error(f"DELETE: {e}")
            raise e

    def devices(self):
        base = "tailf-ncs:devices/device"
        params = "fields=name;address;description;platform"
        ep = f"{base}?{params}"
        return self._get(ep)

    def config(self, dev):
        ep = f"tailf-ncs:devices/device={dev}/config"
        return self._get(ep)

    def packages(self):
        return self._get("tailf-ncs:packages")

    def services(self, service, name=None):
        ep = "tailf-ncs:services"
        if service:
            if name:
                return self._get(f"{ep}/{service}:{service}={name}")
            else:
                return self._get(f"{ep}/{service}:{service}")
        else:
            return self._get(ep)

    def create_service(self, s):
        ep = "tailf-ncs:services"
        data = json.dumps(s.json())
        return self._patch(ep, data)

    def delete_service(self, service, name=None):
        ep = "tailf-ncs:services"
        if service:
            if name:
                return self._delete(f"{ep}/{service}:{service}={name}")
            else:
                return self._delete(f"{ep}/{service}:{service}")
        return self._delete(ep)

    def sync(self, device, action):
        ep = f"tailf-ncs:devices/device={device}/{action}"
        return self._post(ep)




import json
from time import time_ns


IDIPA_SERVICES=["l2ptp", "l2sts"]

class Endpoint():
    def __init__(self, ep):
        if not ep:
            return
        self.device = ep.get("device", None)
        self.port = ep.get("port", None)
        self.ero = ep.get("ero", None)
        self.bw = ep.get("bandwidth", None)
        self.ovlan = ep.get("outer_vlan", None)
        self.ivlan = ep.get("inner_vlan", None)
        self.bsize = ep.get("burst_size", None)
        self.addr_v4 = ep.get("address-v4", None)
        self.mask_v4 = ep.get("netmask-v4", None)
        self.addr_v6 = ep.get("address-v6", None)
        self.mask_v6 = ep.get("netmask-v6", None)
        self.gw_mac = ep.get("gateway-mac-address", None)

        if not self.device:
            raise Exception("No device specified for Endpoint")
        if not self.port:
            raise Exception("No port specified for Endpoint")

    def json_iface(self, device=True):
        ret = {
            "interface": {
                "type": self.port.split()[0],
                "id": self.port.split()[1]
            }
        }
        if device:
            ret["device"] = self.device
        iface = ret["interface"]
        if self.ovlan:
            iface["outervlan"] = self.ovlan
        if self.ivlan:
            iface["innervlan"] = self.ivlan
        if self.bw:
            qos = {"bandwidth": self.bw}
            if self.bsize:
                qos["burst-size"] = self.bsize
            iface.update(qos)
        return ret

    def json_ero(self):
        ret = {"hop": list()}
        if not self.ero:
            return ret
        idx = 1
        for hop in self.ero:
            h = {"address": hop,
                 "index": idx}
            ret["hop"].append(h)
            idx += 1
        return ret

class NSOService():
    def __init__(self, name=None, epa=None, epz=None):
        if name:
            self.name = name
        else:
            now = int(str(time_ns())[-6:])
            self.name = f"fabric-{self.service_str}-{now}"
        # XXX we will need to handle a list of EPs for some services, e.g. l2sts
        self.epa = Endpoint(epa)
        self.epz = Endpoint(epz)

    def __str__(self):
        return json.dumps(self.json())

    def _json_delete(self):
        ret = {
            "tailf-ncs:services": {
                self.service_id: [
                    {
                        "name": self.name
                    }
                ]
            }
        }
        return ret

class idipa(NSOService):
    service_str = "idipa"
    service_id = "idipa:idipa"

    def __init__(self, service=None, name=None):
        self.service = service
        self.name = name

    def json(self, delete=False):
        if delete:
            return self._json_delete()
        ret = {
            "tailf-ncs:services": {
                "idipa:idipa": [
                    {
                        "group": {
                            "service": self.service
                        },
                        "name": self.name
                    }
                ]
            }
        }
        return ret

class l2ptp(NSOService):
    service_str = "l2ptp"
    service_id = "l2ptp:l2ptp"

    def json(self, delete=False):
        if delete:
            return self._json_delete()

        l2ptp = {
            "name": self.name,
            "stp-a": self.epa.json_iface(),
            "stp-z": self.epz.json_iface()
        }

        if self.epa.ero:
            a2z = {"ero-a2z": self.epa.json_ero()}
            l2ptp.update(a2z)

        if self.epz.ero:
            z2a = {"ero-z2a": self.epz.json_ero()}
            l2ptp.update(z2a)

        ret = {
            "tailf-ncs:services": {
                self.service_id: [l2ptp]
            }
        }
        return ret

class l2sts(NSOService):
    service_str = "l2sts"
    service_id = "l2sts:l2sts"

    def json(self, delete=False):
        if delete:
            return self._json_delete()

        l2sts = {
            "name": self.name,
            "site-a": self.epa.json_iface(),
            "site-z": self.epz.json_iface()
        }

        if self.epa.ero:
            a2z = {"ero-a2z": self.epa.json_ero()}
            l2sts.update(a2z)

        if self.epz.ero:
            z2a = {"ero-z2a": self.epz.json_ero()}
            l2sts.update(z2a)

        ret = {
            "tailf-ncs:services": {
                self.service_id: [l2sts]
            }
        }
        return ret

class l2bridge(NSOService):
    service_str = "l2bridge"
    service_id = "l2bridge:l2bridge"

    def json(self, delete=False):
        if delete:
            return self._json_delete()

        l2bridge = {
            "name": self.name,
            "device": self.epa.device,
            "interface": [
                self.epa.json_iface(device=False),
                self.epz.json_iface(device=False)
            ]
        }

        ret = {
            "tailf-ncs:services": {
                self.service_id: [l2bridge]
            }
        }
        return ret

class l3rt(NSOService):
    service_str = "l3rt"
    service_id = "l3rt:l3rt"

    def json(self, delete=False):
        if delete:
            return self._json_delete()

        l3rt = {
            "name": self.name,
            "device": self.epa.device,
            "gateway-mac-address": self.epa.gw_mac,
            "interface": [
                self.epa.json_iface(device=False)["interface"]
            ]
        }

        if self.epa.addr_v4 and self.epa.mask_v4:
            v4 = {
                "gateway-ipv4": {
                    "address": self.epa.addr_v4,
                    "netmask": self.epa.mask_v4
                }
            }
            l3rt.update(v4)

        if self.epa.addr_v6 and self.epa.mask_v6:
            v6 = {
                "gateway-ipv6": {
                    "address": self.epa.addr_v6,
                    "netmask": self.epa.mask_v6
                }
            }
            l3rt.update(v6)

        ret = {
            "tailf-ncs:services": {
                self.service_id: [l3rt]
            }
        }
        return ret

class portmirror(NSOService):
    service_str = "port-mirror"
    service_id = "port-mirror:port-mirror"

    def json(self, delete=False):
        if delete:
            return self._json_delete()

        pm = {
            "name": self.name,
            "device": self.epa.device,
            "from-interface": self.epa.json_iface(device=False)["interface"],
            "to-interface": self.epz.json_iface(device=False)["interface"]
        }

        ret = {
            "tailf-ncs:services": {
                self.service_id: [pm]
            }
        }
        return ret



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric_cf.actor.core.common.constants import Constants


class RuntimeConfig:

    def __init__(self, *, config: list):
        self.kafka_config = {}

        for prop in config:
            for key, value in prop.items():
                self.kafka_config[key] = value

    def get_kafka_config(self):
        return self.kafka_config

    def get_kafka_server(self) -> str:
        return self.kafka_config.get(Constants.PROPERTY_CONF_KAFKA_SERVER, None)

    def get_kafka_schema_registry(self) -> str:
        return self.kafka_config.get(Constants.PROPERTY_CONF_KAFKA_SCHEMA_REGISTRY, None)

    def get_kafka_key_schema(self) -> str:
        return self.kafka_config.get(Constants.PROPERTY_CONF_KAFKA_KEY_SCHEMA, None)

    def get_kafka_value_schema(self) -> str:
        return self.kafka_config.get(Constants.PROPERTY_CONF_KAFKA_VALUE_SCHEMA, None)

    def get_kafka_topic(self) -> str:
        return self.kafka_config.get(Constants.KAFKA_TOPIC, None)

    def get_security_protocol(self) -> str:
        return self.kafka_config.get(Constants.PROPERTY_CONF_KAFKA_SECURITY_PROTOCOL, None)

    def get_group_id(self) -> str:
        return self.kafka_config.get(Constants.PROPERTY_CONF_KAFKA_GROUP_ID, None)

    def get_ca_location(self) -> str:
        return self.kafka_config.get(Constants.PROPERTY_CONF_KAFKA_S_SL_CA_LOCATION, None)

    def get_cert_location(self) -> str:
        return self.kafka_config.get(Constants.PROPERTY_CONF_KAFKA_SSL_CERTIFICATE_LOCATION, None)

    def get_key_location(self) -> str:
        return self.kafka_config.get(Constants.PROPERTY_CONF_KAFKA_SSL_KEY_LOCATION, None)

    def get_key_password(self) -> str:
        return self.kafka_config.get(Constants.PROPERTY_CONF_KAFKA_SSL_KEY_PASSWORD, None)


class LogConfig:
    def __init__(self, *, config: list):
        self.log_dir = None
        self.log_file = None
        self.log_level = None
        self.log_retain = None
        self.log_size = None
        self.log_name = None

        for prop in config:
            for key, value in prop.items():
                if key.lower() == Constants.PROPERTY_CONF_LOG_DIRECTORY:
                    self.log_dir = value
                if key.lower() == Constants.PROPERTY_CONF_LOG_FILE:
                    self.log_file = value
                if key.lower() == Constants.PROPERTY_CONF_LOG_LEVEL:
                    self.log_level = value
                if key.lower() == Constants.PROPERTY_CONF_LOG_RETAIN:
                    self.log_retain = value
                if key.lower() == Constants.PROPERTY_CONF_LOG_SIZE:
                    self.log_size = value
                if key.lower() == Constants.PROPERTY_CONF_LOGGER:
                    self.log_name = value

    def get_log_dir(self) -> str:
        return self.log_dir

    def get_log_file(self) -> str:
        return self.log_file

    def get_log_level(self):
        return self.log_level

    def get_log_retain(self) -> int:
        return self.log_retain

    def get_log_size(self) -> int:
        return self.log_size

    def get_log_name(self) -> int:
        return self.log_name


class AuthConfig:
    def __init__(self, *, config: list):
        self.name = None
        self.guid = None
        self.credmgr_host = None

        for prop in config:
            for key, value in prop.items():
                if key.lower() == Constants.NAME:
                    self.name = value
                if key.lower() == Constants.GUID:
                    self.guid = value
                if key.lower() == Constants.CREDMGR_HOST:
                    self.credmgr_host = value

    def get_name(self) -> str:
        return self.name

    def get_guid(self) -> str:
        return self.guid

    def get_credmgr_host(self) -> str:
        return self.credmgr_host


class Peer:
    def __init__(self, *, config: list):
        self.name = None
        self.type = None
        self.guid = None
        self.kafka_topic = None
        for prop in config:
            for key, value in prop.items():
                if key == Constants.NAME:
                    self.name = value
                elif key == Constants.TYPE:
                    self.type = value
                elif key == Constants.GUID:
                    self.guid = value
                elif key == Constants.KAFKA_TOPIC:
                    self.kafka_topic = value

    def get_name(self) -> str:
        return self.name

    def get_type(self) -> str:
        return self.type

    def get_guid(self) -> str:
        return self.guid

    def get_kafka_topic(self) -> str:
        return self.kafka_topic


class NetConfig:
    def __init__(self, *, config: list):
        self.url = None
        self.username = None
        self.password = None
        self.validate_certs = None

        for prop in config:
            for key, value in prop.items():
                if key == "url":
                    self.url = value
                elif key == "username":
                    self.username = value
                elif key == "password":
                    self.password = value
                elif key == "validate_certs":
                    self.validate_certs = value

    def get_url(self) -> str:
        return self.url

    def get_username(self) -> str:
        return self.username

    def get_password(self) -> str:
        return self.password

    def get_validate_certs(self) -> bool:
        return self.validate_certs


class Configuration:
    def __init__(self, config: dict):
        self.runtime = RuntimeConfig(config=config[Constants.CONFIG_SECTION_RUNTIME])
        self.logging = LogConfig(config=config[Constants.CONFIG_LOGGING_SECTION])
        self.auth = AuthConfig(config=config['auth'])
        self.net = NetConfig(config=config['net'])
        self.peers = []
        if 'peers' in config:
            for e in config['peers']:
                self.peers.append(Peer(config=e['peer']))

    def get_runtime_config(self) -> RuntimeConfig:
        return self.runtime

    def get_peers(self) -> list:
        return self.peers

    def get_auth(self) -> AuthConfig:
        return self.auth

    def get_logging(self) -> LogConfig:
        return self.logging

    def get_net(self) -> NetConfig:
        return self.net



import yaml

class Inventory:
    def __init__(self, yfile):
        self.nodes = dict()
        self.paths = dict()
        with open(yfile, "r") as inv:
            try:
                data = yaml.safe_load(inv)
                for k,v in data.items():
                    if isinstance(v, dict):
                        if (k == "paths"):
                            self.paths.update(v)
                        if (k == "nodes"):
                            self.nodes.update(v)
            except Exception as e:
                raise Exception(f"Could not load configuration file: {ifile}: {e}")
            inv.close()

    def resolve_resource(self, rname):
        if rname not in self.nodes:
            raise Exception(f"Node not found: {rname}")
        ret = self.nodes.get(rname)
        path = ret.get("path", None)
        if not path:
            ret["ero"] = None
        else:
            if path not in self.paths:
                raise Exception(f"Path not found: {rname} -> {path}")
            ret["ero"] = self.paths.get(path)
        return ret




#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import json
import traceback
from typing import Tuple, List

from fabric_cf.actor.core.apis.abc_delegation import DelegationState
from fabric_cf.actor.core.common.constants import Constants
from fabric_cf.actor.core.kernel.reservation_states import ReservationStates, ReservationPendingStates
from fabric_cf.actor.core.kernel.slice_state_machine import SliceState
from fabric_cf.actor.core.manage.error import Error
from fabric_cf.actor.core.time.actor_clock import ActorClock
from fabric_cf.actor.core.util.id import ID
from fabric_cf.actor.core.util.utils import sliver_to_str
from fabric_mb.message_bus.messages.delegation_avro import DelegationAvro
from fabric_mb.message_bus.messages.reservation_mng import ReservationMng
from fabric_mb.message_bus.messages.slice_avro import SliceAvro
from fim.graph.abc_property_graph import ABCPropertyGraph
from fim.slivers.network_node import NodeSliver
from fim.slivers.network_service import NetworkServiceSliver

from fabric_mgmt_cli.managecli.command import Command


class ShowCommand(Command):
    def get_slices(self, *, actor_name: str, callback_topic: str, slice_id: str, slice_name: str, id_token: str,
                   email: str, states: str, format: str):
        try:
            slices, error = self.do_get_slices(actor_name=actor_name, callback_topic=callback_topic, slice_id=slice_id,
                                               slice_name=slice_name, id_token=id_token, email=email, states=states)
            if slices is not None and len(slices) > 0:
                self.__print_slices(slices=slices, format=format)
            else:
                print("Status: {}".format(error.get_status()))
        except Exception as e:
            ex_str = traceback.format_exc()
            self.logger.error(ex_str)
            print("Exception occurred while processing get_slices {}".format(e))

    def get_reservations(self, *, actor_name: str, callback_topic: str, slice_id: str, rid: str,
                         states: str, id_token: str, email: str, site:str, type: str, format: str, fields: str):
        try:
            reservations, error = self.do_get_reservations(actor_name=actor_name, callback_topic=callback_topic,
                                                           slice_id=slice_id, rid=rid, states=states, id_token=id_token,
                                                           email=email, site=site, type=type)
            if reservations is not None and len(reservations) > 0:
                self.__print_reservations(reservations=reservations, format=format, fields=fields)
            else:
                print("Status: {}".format(error.get_status()))
        except Exception as e:
            ex_str = traceback.format_exc()
            self.logger.error(ex_str)
            print("Exception occurred while processing get_reservations {}".format(e))

    def get_delegations(self, *, actor_name: str, callback_topic: str, slice_id: str, did: str, state: str,
                        id_token: str, format: str):
        try:
            delegations, error = self.do_get_delegations(actor_name=actor_name, callback_topic=callback_topic,
                                                         slice_id=slice_id, did=did, state=state, id_token=id_token)
            if delegations is not None and len(delegations) > 0:
                self.__print_delegations(delegations=delegations, format=format)
            else:
                print("Status: {}".format(error.get_status()))
        except Exception as e:
            ex_str = traceback.format_exc()
            self.logger.error(ex_str)
            print("Exception occurred while processing get_delegations {}".format(e))

    def do_get_slices(self, *, actor_name: str, callback_topic: str, slice_id: str = None, slice_name: str = None,
                      id_token: str = None, email: str = None, states: str = None) -> Tuple[List[SliceAvro] or None, Error]:
        actor = self.get_actor(actor_name=actor_name)

        if actor is None:
            raise Exception("Invalid arguments actor {} not found".format(actor_name))
        try:
            actor.prepare(callback_topic=callback_topic)
            sid = ID(uid=slice_id) if slice_id is not None else None
            slice_state = None
            if states is not None:
                states_list = states.split(",")
                for x in states_list:
                    if slice_state is None:
                        slice_state = []
                    slice_state.append(SliceState.translate(state_name=x).value)

            result = actor.get_slices(slice_id=sid, slice_name=slice_name, email=email, state=slice_state)
            return result, actor.get_last_error()
        except Exception:
            ex_str = traceback.format_exc()
            self.logger.error(ex_str)
        return None, actor.get_last_error()

    def do_get_reservations(self, *, actor_name: str, callback_topic: str, slice_id: str = None, rid: str = None,
                            states: str = None, id_token: str = None, email: str = None, site: str = None,
                            type: str = None) -> Tuple[List[ReservationMng] or None, Error]:
        actor = self.get_actor(actor_name=actor_name)

        if actor is None:
            raise Exception("Invalid arguments actor {} not found".format(actor_name))
        try:
            actor.prepare(callback_topic=callback_topic)
            sid = ID(uid=slice_id) if slice_id is not None else None
            reservation_id = ID(uid=rid) if rid is not None else None
            reservation_state = None
            if states is not None and states != "all":
                states_list = states.split(",")
                for x in states_list:
                    if reservation_state is not None:
                        reservation_state = []
                    reservation_state.append(ReservationStates.translate(state_name=x).value)
            return actor.get_reservations(slice_id=sid, rid=reservation_id, states=reservation_state, email=email,
                                          site=site, type=type), actor.get_last_error()
        except Exception as e:
            ex_str = traceback.format_exc()
            self.logger.error(ex_str)
        return None, actor.get_last_error()

    def do_get_delegations(self, *, actor_name: str, callback_topic: str, slice_id: str = None, did: str = None,
                           state: str = None, id_token: str = None) -> Tuple[List[DelegationAvro] or None, Error]:
        actor = self.get_actor(actor_name=actor_name)

        if actor is None:
            raise Exception("Invalid arguments actor {} not found".format(actor_name))
        try:
            actor.prepare(callback_topic=callback_topic)
            sid = None
            if slice_id is not None:
                sid = ID(uid=slice_id)
            delegation_state = None
            if state is not None and state != "all":
                delegation_state = [DelegationState.translate(state_name=state).value]
            return actor.get_delegations(delegation_id=did, slice_id=sid,
                                         state=delegation_state), actor.get_last_error()
        except Exception as e:
            self.logger.error(f"Exception occurred while fetching delegations: e {e}")
            self.logger.error(traceback.format_exc())
            traceback.print_exc()
        return None, actor.get_last_error()

    @staticmethod
    def __print_reservations_json(*, reservations: List[ReservationMng], fields: str):
        res_list = []
        if fields is not None:
            field_list = fields.split(",")
        else:
            field_list = None
        for reservation in reservations:
            res_dict = {
                'sliver_id': reservation.reservation_id,
                'slice_id': reservation.slice_id
            }
            if reservation.rtype is not None and (field_list is None or 'type' in field_list):
                res_dict['type'] = reservation.rtype

            if reservation.rtype is not None and (field_list is None or 'notices' in field_list):
                res_dict['notices'] = reservation.notices

            if reservation.start is not None and (field_list is None or 'start' in field_list):
                res_dict['start'] = ShowCommand.__time_string(milliseconds=reservation.start)

            if reservation.end is not None and (field_list is None or 'end' in field_list):
                res_dict['end'] = ShowCommand.__time_string(milliseconds=reservation.end)

            if reservation.requested_end is not None and (field_list is None or 'requested_end' in field_list):
                res_dict['requested_end'] = ShowCommand.__time_string(milliseconds=reservation.requested_end)

            if reservation.units is not None and (field_list is None or 'units' in field_list):
                res_dict['units'] = reservation.units

            if reservation.state is not None and (field_list is None or 'state' in field_list):
                res_dict['state'] = reservation.state

            if reservation.pending_state is not None and (field_list is None or 'pending_state' in field_list):
                res_dict['pending_state'] = reservation.pending_state

            sliver = reservation.get_sliver()
            if sliver is not None and (field_list is None or 'sliver' in field_list):
                res_dict['sliver'] = ABCPropertyGraph.sliver_to_dict(sliver)

            res_list.append(res_dict)

        print(json.dumps(res_list, indent=4))

    def __print_reservations(self, reservations: List[ReservationMng], format: str, fields: str):
        if format == 'text':
            for r in reservations:
                self.__print_reservation(reservation=r)
        else:
            self.__print_reservations_json(reservations=reservations, fields=fields)

    @staticmethod
    def __print_reservation(*, reservation: ReservationMng):
        """
        Prints ReservationMng
        """
        print("")
        print(f"Reservation ID: {reservation.reservation_id} Slice ID: {reservation.slice_id}")
        if reservation.rtype is not None or reservation.notices is not None:
            print(f"Resource Type: {reservation.rtype} Notices: {reservation.notices}")

        if reservation.start is not None or reservation.end is not None or reservation.requested_end is not None:
            print(f"Start: {ShowCommand.__time_string(milliseconds=reservation.start)} "
                  f"End: {ShowCommand.__time_string(milliseconds=reservation.end)} "
                  f"Requested End: {ShowCommand.__time_string(milliseconds=reservation.requested_end)}")

        if reservation.units is not None or reservation.state is not None or reservation.pending_state is not None:
            print(f"Units: {reservation.units} State: {ReservationStates(reservation.state)} "
                  f"Pending State: {ReservationPendingStates(reservation.pending_state)}")

        sliver = reservation.get_sliver()
        if sliver is not None:
            print(f"Sliver: {sliver_to_str(sliver=sliver)}")
        print("")

    @staticmethod
    def __print_node_sliver(*, sliver: NodeSliver):
        if sliver.attached_components_info is not None:
            for c in sliver.attached_components_info.devices.items():
                print(c)

    @staticmethod
    def __print_ns_sliver(*, sliver: NetworkServiceSliver):
        if sliver.interface_info is not None:
            for c in sliver.interface_info.interfaces.values():
                print(c)

    @staticmethod
    def __time_string(*, milliseconds):
        time_obj = ActorClock.from_milliseconds(milli_seconds=milliseconds)
        return time_obj.strftime(Constants.LEASE_TIME_FORMAT)

    @staticmethod
    def __print_slice(*, slice_object: SliceAvro):
        """
        Prints Slice Object
        """
        print("")
        print(f"Slice Name: {slice_object.get_slice_name()} Slice ID: {slice_object.get_slice_id()} "
              f"Project ID: {slice_object.get_slice_id()}")
        if slice_object.get_graph_id() is not None:
            print(f"Graph ID: {slice_object.get_graph_id()}")

        if slice_object.get_owner() is not None:
            print(f"Slice owner: {slice_object.get_owner()}")

        if slice_object.get_state() is not None:
            print(f"Slice state: {str(SliceState(slice_object.get_state()))}")

        if slice_object.get_lease_end() is not None:
            print(f"Lease time: {slice_object.get_lease_end()}")
        print("")

    @staticmethod
    def __print_slice_json(*, slices: List[SliceAvro]):
        """
        Prints Slice Object
        """
        slc_list = []
        for slice_object in slices:
            slc_dict = {'name': slice_object.get_slice_name(),
                        'slice_id': slice_object.get_slice_id(),
                        'project_id': slice_object.get_project_id(),
                        'graph_id': slice_object.get_graph_id(),
                        'owner': slice_object.get_owner().get_email(),
                        'state': str(SliceState(slice_object.get_state())),
                        'lease_start_time': str(slice_object.get_lease_start()),
                        'lease_end_time': str(slice_object.get_lease_end())
                        }
            slc_list.append(slc_dict)

        print(json.dumps(slc_list, indent=4))

    def __print_slices(self, slices: List[SliceAvro], format: str):
        if format == 'text':
            for s in slices:
                self.__print_slice(slice_object=s)
        else:
            self.__print_slice_json(slices=slices)

    @staticmethod
    def __print_delegation(*, dlg_object: DelegationAvro):
        """
        Prints the Delegation Object
        """
        print("")
        print("Delegation ID: {} Slice ID: {}".format(dlg_object.delegation_id, dlg_object.slice.get_slice_id()))
        if dlg_object.delegation_name is not None:
            print("Delegation Name: {}".format(dlg_object.delegation_name))
        if dlg_object.sequence is not None:
            print("Sequence: {}".format(dlg_object.sequence))
        if dlg_object.state is not None:
            print(f"State: {DelegationState(dlg_object.state)}")
        if dlg_object.graph is not None:
            print("Graph: {}".format(dlg_object.graph))
        print("")

    @staticmethod
    def __print_delegations_json(*, delegations: List[DelegationAvro]):
        """
        Prints the Delegation Object
        """
        dlg_list = []
        for dlg_object in delegations:
            dlg_dict = {
                'name': dlg_object.get_name(),
                'dlg_id': dlg_object.get_delegation_id(),
                'slice_id': dlg_object.get_delegation_id(),
                'sequence': dlg_object.get_sequence(),
                'state': str(DelegationState(dlg_object.state)),
                'graph': dlg_object.graph
            }
            dlg_list.append(dlg_dict)
        print(json.dumps(dlg_list, indent=4))

    def __print_delegations(self, *, delegations: List[DelegationAvro], format: str):
        if format == 'text':
            for d in delegations:
                self.__print_delegation(dlg_object=d)
        else:
            self.__print_delegations_json(delegations=delegations)


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import logging
import os
import threading
import traceback
from logging.handlers import RotatingFileHandler

from fabric_cf.actor.core.apis.abc_actor_mixin import ActorType
from fabric_cf.actor.core.manage.kafka.kafka_actor import KafkaActor
from fabric_cf.actor.core.manage.kafka.kafka_broker import KafkaBroker
from fabric_cf.actor.core.manage.kafka.kafka_mgmt_message_processor import KafkaMgmtMessageProcessor
from fabric_cf.actor.core.util.id import ID
from fabric_cm.credmgr.credmgr_proxy import CredmgrProxy

from fabric_mgmt_cli.managecli.config_processor import ConfigProcessor


class TokenException(Exception):
    pass


class KafkaProcessor:
    PATH = os.environ.get('FABRIC_MGMT_CLI_CONFIG_PATH', './config.yml')

    def __init__(self):
        self.config_processor = ConfigProcessor(path=self.PATH)
        self.message_processor = None
        self.actor_cache = {}
        self.lock = threading.Lock()
        self.auth = None
        self.logger = None
        self.key_schema = None
        self.val_schema = None
        self.producer = None

    def setup_kafka(self):
        """
        Set up Kafka Producer and Consumer
        """
        conf = self.config_processor.get_kafka_config_producer()
        self.key_schema = self.config_processor.get_kafka_key_schema()
        self.val_schema = self.config_processor.get_kafka_value_schema()

        from fabric_mb.message_bus.producer import AvroProducerApi
        self.producer = AvroProducerApi(producer_conf=conf, key_schema_location=self.key_schema,
                                        value_schema_location=self.val_schema, logger=self.logger)

        consumer_conf = self.config_processor.get_kafka_config_consumer()
        topics = [self.config_processor.get_kafka_topic()]

        self.message_processor = KafkaMgmtMessageProcessor(consumer_conf=consumer_conf,
                                                           key_schema_location=self.key_schema,
                                                           value_schema_location=self.val_schema, topics=topics,
                                                           logger=self.logger)

    def initialize(self):
        """
        Initialize the Kafka Processor
        """
        self.config_processor.process()

        self.logger = self.make_logger()

        self.setup_kafka()

        self.load_actor_cache()

    def load_actor_cache(self):
        """
        Load the Actor Cache
        """
        peers = self.config_processor.get_peers()
        if peers is not None:
            for p in peers:
                mgmt_actor = None
                if p.get_type().lower() == ActorType.Broker.name.lower():
                    mgmt_actor = KafkaBroker(guid=ID(uid=p.get_guid()), kafka_topic=p.get_kafka_topic(),
                                             auth=self.config_processor.get_auth(),
                                             logger=self.logger, message_processor=self.message_processor,
                                             producer=self.producer)
                else:
                    mgmt_actor = KafkaActor(guid=ID(uid=p.get_guid()), kafka_topic=p.get_kafka_topic(),
                                            auth=self.config_processor.get_auth(),
                                            logger=self.logger, message_processor=self.message_processor,
                                            producer=self.producer)
                try:
                    self.lock.acquire()
                    self.logger.debug("Added actor {} to cache".format(p.get_name()))
                    self.actor_cache[p.get_name()] = mgmt_actor
                finally:
                    self.lock.release()
        else:
            self.logger.debug("No peers available")

    def get_mgmt_actor(self, *, name: str) -> KafkaActor:
        """
        Get Management Actor from Cache
        @param name actor name
        @return Management Actor
        """

        try:
            self.lock.acquire()
            return self.actor_cache.get(name, None)
        finally:
            self.lock.release()

    def make_logger(self):
        """
        Detects the path and level for the log file from the actor config and sets
        up a logger. Instead of detecting the path and/or level from the
        config, a custom path and/or level for the log file can be passed as
        optional arguments.

        :param log_path: Path to custom log file
        :param log_level: Custom log level
        :return: logging.Logger object
        """

        # Get the log path
        if self.config_processor is None:
            raise RuntimeError('No config information available')

        log_path = self.config_processor.get_log_dir() + '/' + self.config_processor.get_log_file()

        if log_path is None:
            raise RuntimeError('The log file path must be specified in config or passed as an argument')

        # Get the log level
        log_level = self.config_processor.get_log_level()
        if log_level is None:
            log_level = logging.INFO

        # Set up the root logger
        log = logging.getLogger(self.config_processor.get_log_name())
        log.setLevel(log_level)
        log_format = \
            '%(asctime)s - %(name)s - {%(filename)s:%(lineno)d} - [%(threadName)s] - %(levelname)s - %(message)s'

        os.makedirs(os.path.dirname(log_path), exist_ok=True)

        backup_count = self.config_processor.get_log_retain()
        max_log_size = self.config_processor.get_log_size()

        file_handler = RotatingFileHandler(log_path, backupCount=int(backup_count), maxBytes=int(max_log_size))

        logging.basicConfig(handlers=[file_handler], format=log_format)

        return log

    def get_callback_topic(self) -> str:
        """
        Get Callback Topic
        @return callback topic
        """
        return self.config_processor.get_kafka_topic()

    def get_tokens(self, id_token: str, refresh_token: str) -> str:
        """
        Get Tokens
        @param id_token id token
        @param refresh_token refresh token
        @return Fabric Identity Token
        """
        tokens = None
        if refresh_token is None:
            refresh_token = os.getenv('FABRIC_REFRESH_TOKEN', None)

        if refresh_token is not None:
            try:
                proxy = CredmgrProxy(credmgr_host=self.config_processor.get_credmgr_host())
                tokens = proxy.refresh(project_name="all", scope="all", refresh_token=refresh_token)
                id_token = tokens.get('id_token', None)
            except Exception as e:
                raise TokenException('Not a valid refresh_token! Error: {}'.format(e))

        if id_token is None:
            id_token = os.getenv('FABRIC_ID_TOKEN', None)
            raise TokenException('Either id_token or refresh_token must be specified! Alternatively, '
                                 'set the environment variables FABRIC_ID_TOKEN and FABRIC_REFRESH_TOKEN')

        return id_token

    def start(self, id_token: str, refresh_token: str, ignore_tokens: bool = False) -> str:
        """
        Start Synchronous Kafka Processor
        @param id_token identity token
        @param refresh_token refresh token
        @param ignore_tokens flag  to indicate to ignore tokens
        @return token if ignore_tokens is False; None otherwise
        """
        try:
            self.initialize()
            ret_val = None
            if not ignore_tokens:
                ret_val = self.get_tokens(id_token=id_token, refresh_token=refresh_token)
            self.message_processor.start()
            return ret_val
        except TokenException as e:
            self.logger.debug(f"Failed to start Management Shell: {e}")
            self.logger.error(traceback.format_exc())
            raise e
        except Exception as e:
            self.logger.debug(f"Failed to start Management Shell: {e}")
            self.logger.error(traceback.format_exc())
            raise e

    def stop(self):
        """
        Stop the Synchronous Kafka Processor
        """
        try:
            self.message_processor.stop()
        except Exception as e:
            self.logger.debug(f"Failed to stop Management Shell: {e}")
            self.logger.error(traceback.format_exc())


class KafkaProcessorSingleton:
    """
    Kafka Processor Singleton
    """
    __instance = None

    def __init__(self):
        if self.__instance is not None:
            raise Exception("Singleton can't be created twice !")

    def get(self):
        """
        Actually create an instance
        """
        if self.__instance is None:
            self.__instance = KafkaProcessor()
        return self.__instance

    get = classmethod(get)
